@@ -3,7 +3,6 @@ import logging
 import re
 import shutil
 import subprocess
-import sys
 import threading
 from http.server import SimpleHTTPRequestHandler
 from http.server import test
@@ -18,20 +17,23 @@ from jinja2 import PackageLoader
 from jinja2.exceptions import TemplateNotFound
 from slugify import slugify
 from pylingdocs.config import BENCH
+from pylingdocs.config import CONTENT_FILE_PREFIX
 from pylingdocs.config import CONTENT_FOLDER
 from pylingdocs.config import DATA_DIR
 from pylingdocs.config import FIGURE_DIR
-from pylingdocs.config import GLOSS_ABBREVS
+from pylingdocs.config import GLOSS_FILE_ADDRESS
 from pylingdocs.config import LATEX_TOPLEVEL
 from pylingdocs.config import OUTPUT_DIR
 from pylingdocs.config import OUTPUT_TEMPLATES
 from pylingdocs.config import STRUCTURE_FILE
 from pylingdocs.helpers import _get_relative_file
-from pylingdocs.helpers import _load_structure
+from pylingdocs.helpers import _load_cldf_dataset
 from pylingdocs.helpers import decorate_gloss_string
+from pylingdocs.helpers import get_structure
 from pylingdocs.helpers import html_example_wrap
 from pylingdocs.helpers import html_gloss
 from pylingdocs.helpers import latexify_table
+from pylingdocs.helpers import load_content
 from pylingdocs.helpers import refresh_clld_db
 from pylingdocs.helpers import src
 from pylingdocs.metadata import _load_metadata
@@ -43,6 +45,7 @@ from pylingdocs.preprocessing import render_markdown
 
 
 NUM_PRE = re.compile(r"[\d]+\ ")
+ABC_PRE = re.compile(r"[A-Z]+\ ")
 
 log = logging.getLogger(__name__)
 
@@ -410,8 +413,9 @@ class CLLD(OutputFormat):
             if len(df) == 0:
                 df = df.append({x: "" for x in df.columns}, ignore_index=True)
             return df.to_markdown(index=False)
+        cap = "".join(cls.replace_commands(caption))
         return (
-            f"<a id='tab:{label}'></a><div class='caption table' id='tab:{label}'>{caption}</div>\n\n"
+            f"<a id='tab:{label}'></a><div class='caption table' id='tab:{label}'>{cap}</div>\n\n"
             + df.to_markdown(index=False)
         )
 
@@ -441,6 +445,7 @@ class CLLD(OutputFormat):
         if len(parts) == 0 or OUTPUT_TEMPLATES["clld"] in ["slides", "article"]:
             # these use # as section markers, so we add a level for the html output
             tent = tent.replace("\n#", "\n##")
+            tent = f"# {metadata['title']}\n\n" + tent
             with open(my_output_dir / "content.txt", "w", encoding="utf-8") as f:
                 f.write(tent)
         else:
@@ -642,26 +647,11 @@ class Latex(OutputFormat):
 builders = {x.name: x for x in [PlainText, GitHub, Latex, HTML, CLLD]}
 
 
-def _iterate_structure(structure, level, depths):
-    for child_id, child_data in structure.items():
-        depths[level] += 1
-        yield child_id, level, child_data["title"], "".join(
-            [str(x) for x in depths.values()]
-        )
-        if isinstance(child_data, dict) and "parts" in child_data:
-            for x in _iterate_structure(child_data["parts"], level + 1, depths):
-                yield x
-
-
-def iterate_structure(structure):
-    for x in _iterate_structure(
-        structure["document"]["parts"], level=0, depths={0: 0, 1: 0, 2: 0, 3: 0}
-    ):
-        yield x
-
-
 def update_structure(
-    content_dir=CONTENT_FOLDER, bench_dir=BENCH, structure_file=STRUCTURE_FILE
+    content_dir=CONTENT_FOLDER,
+    bench_dir=BENCH,
+    structure_file=STRUCTURE_FILE,
+    prefix_mode=CONTENT_FILE_PREFIX,
 ):
     log.info("Updating document structure")
 
@@ -670,24 +660,25 @@ def update_structure(
         if ".md" not in file.name:
             continue
         name = re.sub(NUM_PRE, "", file.stem)
+        name = re.sub(ABC_PRE, "", name)
         content_files[name] = file
 
     bench_files = {}
-    for file in bench_dir.iterdir():
-        if ".md" not in file.name:
-            continue
-        name = re.sub(NUM_PRE, "", file.stem)
-        bench_files[name] = file
+    if Path(bench_dir).is_dir():
+        for file in bench_dir.iterdir():
+            if ".md" not in file.name:
+                continue
+            name = re.sub(NUM_PRE, "", file.stem)
+            name = re.sub(ABC_PRE, "", name)
+            bench_files[name] = file
 
-    structure = _load_structure(
-        _get_relative_file(folder=content_dir, file=structure_file)
+    structure = get_structure(
+        prefix_mode=prefix_mode,
+        structure_file=_get_relative_file(content_dir, structure_file),
     )
 
-    for part_id, level, title, fileno in iterate_structure(structure):
-        del level  # unused
-        del title  # unused
-        fname = f"{fileno} {part_id}.md"
-        new_path = Path(content_dir, fname)
+    for part_id, data in structure.items():
+        new_path = Path(content_dir, data["filename"])
         if part_id in content_files:
             if part_id in bench_files:
                 log.warning(f"Conflict: {part_id}. Resolve manually.")
@@ -722,24 +713,6 @@ def compile_latex(output_dir=OUTPUT_DIR):  # pragma: no cover
         del proc  # help, prospector is forcing me
 
 
-def _load_content(structure, source_dir=CONTENT_FOLDER):
-    source_dir = Path(source_dir)
-    contents = {}
-    parts = {}
-    for part_id, level, title, fileno in iterate_structure(structure):
-        del level  # unused
-        filename = f"{fileno} {part_id}.md"
-        try:
-            with open(source_dir / filename, "r", encoding="utf-8") as f:
-                content = f.read()
-        except FileNotFoundError:
-            log.error(f"File {(source_dir/filename).resolve()} does not exist.")
-            sys.exit(1)
-        contents[part_id] = content
-        parts[part_id] = title
-    return contents, parts
-
-
 class Handler(SimpleHTTPRequestHandler):
     def __init__(self, *args, **kwargs):
         super().__init__(*args, directory=str(OUTPUT_DIR / "html"), **kwargs)
@@ -749,10 +722,21 @@ def run_server():
     test(Handler)
 
 
-def run_preview(contents, refresh=True, **kwargs):
+def run_preview(cldf, source_dir, output_dir, refresh=True, **kwargs):
     log.info("Rendering preview")
-    watchfiles = [str(x) for x in kwargs["source_dir"].iterdir()]
-    watchfiles += [str(x) for x in (kwargs["source_dir"] / CONTENT_FOLDER).iterdir()]
+    watchfiles = [str(x) for x in source_dir.iterdir()]
+    watchfiles += [str(x) for x in (source_dir / CONTENT_FOLDER).iterdir()]
+    ds = _load_cldf_dataset(cldf)
+    structure_file = _get_relative_file(
+        folder=source_dir / CONTENT_FOLDER, file=STRUCTURE_FILE
+    )
+    contents = load_content(
+        structure_file=structure_file, source_dir=source_dir / CONTENT_FOLDER
+    )
+    kwargs["cldf"] = cldf
+    kwargs["dataset"] = ds
+    kwargs["source_dir"] = source_dir
+    kwargs["output_dir"] = output_dir
     if refresh:
         wkwargs = kwargs.copy()
         reloader = hupper.start_reloader(
@@ -787,35 +771,20 @@ def _write_file(part_id):
     log.debug(f"Writing {part_id}")
 
 
-def check_ids(source_dir, dataset, structure):
-
-    if isinstance(structure, (str, PosixPath)):
-        structure = _load_structure(source_dir / CONTENT_FOLDER / structure)
-
-    source_dir = Path(source_dir)
-    contents, parts = _load_content(structure, source_dir / CONTENT_FOLDER)
-    del parts
-
+def check_ids(contents, dataset, source_dir):
     builder = builders["plain"]
-    content = "\n\n".join(contents.values())
-    preprocessed = preprocess(content, source_dir)
-    preprocessed = builder.preprocess_commands(preprocessed)
-    running = True
-    bad_ids = []
-    while running:
-        try:
-            render_markdown(preprocessed, dataset, output_format="plain")
-        except KeyError as e:
-            bad_id = str(e).strip("'")
-            preprocessed = preprocessed.replace(bad_id, "")
-            bad_ids.append(bad_id)
-        else:
-            running = False
-    if len(bad_ids) > 0:
-        bad_ids = "\n".join(bad_ids)
-        log.error(f"""IDs missing from the dataset:\n{bad_ids}""")
-    else:
-        log.info("All good!")
+    found = False
+    for filename, x in contents.items():
+        preprocessed = preprocess(x["content"], source_dir)
+        preprocessed = builder.preprocess_commands(preprocessed)
+        for i, line in enumerate(preprocessed.split("\n")):
+            try:
+                render_markdown(line, dataset, output_format="plain")
+            except KeyError as e:
+                log.error(f"Missing ID in file {filename}, L{i+1}:\n{str(e)} in {line}")
+                found = True
+    if not found:
+        log.info("No missing IDs found.")
 
 
 def create_output(
@@ -849,6 +818,14 @@ def create_output(
     if not output_dir.is_dir():
         log.info(f"Creating output folder {output_dir.resolve()}")
         output_dir.mkdir()
+    if "cldf:" in GLOSS_FILE_ADDRESS:
+        gloss_dict = {}
+    elif (Path(source_dir) / GLOSS_FILE_ADDRESS).is_file():
+        df = pd.read_csv(Path(source_dir) / GLOSS_FILE_ADDRESS)
+        gloss_dict = dict(zip(df["ID"].str.lower(), df["Description"]))
+    else:
+        gloss_dict = {}
+
     for output_format in formats:
         for m in models:
             m.reset_cnt()
@@ -876,14 +853,14 @@ def create_output(
                 output_dir,
                 content=preprocessed,
                 metadata=metadata,
-                abbrev_dict=GLOSS_ABBREVS,
+                abbrev_dict=gloss_dict,
             )
         elif builder.name == "clld":
             builder.write_folder(
                 output_dir,
                 content=preprocessed,
                 metadata=metadata,
-                abbrev_dict=GLOSS_ABBREVS,
+                abbrev_dict=gloss_dict,
             )
             # builder.create_app()
             # write_clld(preprocessed)
